@@ -1,8 +1,5 @@
-import sys
 import logging
-import itertools
 import importlib
-import traceback
 
 import asyncio
 import aiohttp
@@ -25,33 +22,29 @@ class Hypotonic:
     module = importlib.import_module('hypotonic.command')
     while True:
       commands, context, data = await queue.get()
+      command, args, kwargs = commands.pop()
 
       try:
-        # All commands were executed succesfully.
-        if not commands:
-          self.results.append(data)
-          continue
-
-        command, args, kwargs = commands.pop()
         logger.debug(("Start", i, command, args, kwargs))
 
         # Dynamically load the command function.
         func = getattr(module, command)
-        async for result in func(session, context, data, *args, **kwargs):
-          # TODO Passing commands by value is not efficient.
-          commands_copy = list(commands)
-          logger.debug(("Queue", i, commands_copy, []), *result)
-          queue.put_nowait((commands_copy, *result))
+        async for context, data in func(session, context, data, *args, **kwargs):
+          if not commands:
+            self.results.append(data)
+          else:
+            logger.debug(("Queue", i, commands, context, data))
+            queue.put_nowait((commands.copy(), context, data))
 
         logger.debug(("Stop", i, command, args, kwargs))
-      except:
-        self.errors.append(((command, args, kwargs, context), sys.exc_info()))
-        logger.info(traceback.format_exc())
+      except Exception as error:
+        self.errors.append(((command, args, kwargs), context, error))
+        logger.debug(('Error', i, command, args, kwargs, error))
       finally:
         queue.task_done()
 
   async def run(self):
-    session = aiohttp.ClientSession()
+    session = aiohttp.ClientSession(raise_for_status=True)
     queue = asyncio.Queue()
 
     tasks = []
